@@ -14,11 +14,9 @@ print "\nneed a version number as arg\n" if ! $ver;
 
 my $des = Devel::Examine::Subs->new(
     file => 'lib/Devel/Examine',
-    extensions => ['pm'],
+    extensions => ['*.pm'],
 );
 
 my $cref = sub { $_[0] =~ s/(our \$VERSION =).*/$1 '$ver';/; };
 
 my $ret = $des->replace(exec => $cref, limit => 1);
-
-print Dumper $ret;
