@@ -3,7 +3,7 @@ use 5.012;
 use warnings;
 use strict;
 
-our $VERSION = '1.50';
+our $VERSION = '1.51';
 
 use Carp;
 use Data::Compare;
@@ -14,7 +14,6 @@ use Devel::Examine::Subs::Postprocessor;
 use File::Basename;
 use File::Copy;
 use File::Edit::Portable;
-use File::Find;
 use PPI;
 use Symbol qw(delete_package);
 
@@ -54,7 +53,7 @@ sub new {
 
     # default configs
 
-    $self->{namespace} = 'Devel::Examine::Subs';
+    $self->{namespace} = __PACKAGE__;
     $self->{params}{regex} = 1;
 
     $self->_config($p);
@@ -203,7 +202,7 @@ sub inject {
 
     # inject_use/inject_after_sub_def are preprocs
 
-    if ($p->{inject_use} || $p->{inject_after_sub_def}){
+    if ($p->{inject_use} || $p->{inject_after_sub_def} || $p->{line_num}){
         $self->{params}{pre_proc} = 'inject';
         $self->{params}{pre_proc_return} = 1;
     }
@@ -246,18 +245,39 @@ sub add_functionality {
                     engine
     );
 
+    my $is_allowed = 0;
 
-    if (! (grep {$to_add eq $_} @allowed)){
+    for (@allowed){
+        if ($_ eq $to_add){
+            $is_allowed = 1;
+            last;
+        }
+    }
+    
+    if (! $is_allowed){
         croak "Adding a non-allowed piece of functionality...\n";
     }
 
     my %dt = (
+            pre_proc => sub {
+                        trace() if $ENV{TRACE};
+                        return $in_prod
+                        ? $INC{'Devel/Examine/Subs/Preprocessor.pm'}
+                        : 'lib/Devel/Examine/Subs/Preprocessor.pm';
+                      },
+            post_proc => sub {
+                        trace() if $ENV{TRACE};
+                        return $in_prod
+                        ? $INC{'Devel/Examine/Subs/Postprocessor.pm'}
+                        : 'lib/Devel/Examine/Subs/Postprocessor.pm';
+                      },
             engine => sub {
                         trace() if $ENV{TRACE};
                         return $in_prod
                         ? $INC{'Devel/Examine/Subs/Engine.pm'}
                         : 'lib/Devel/Examine/Subs/Engine.pm';
                       },
+
     );
 
     my $caller = (caller)[1];
@@ -281,12 +301,41 @@ sub add_functionality {
 
     my $file = $dt{$to_add}->();
 
-    my $des = Devel::Examine::Subs->new({
+    my $des = Devel::Examine::Subs->new(
                                     file => $file,
-                                    post_proc => 'end_of_last_sub',
-                                });
+                                    engine => 'objects', 
+                                    post_proc => [qw(subs end_of_last_sub)],
+                                );
+    $p = {
+        engine => 'objects', 
+        post_proc => [qw(subs end_of_last_sub)],
+        post_proc_return => 1,
+    };
 
-    $p->{write_file_contents} = \@code;
+    my $start_writing = $des->run($p);
+
+    my $rw = File::Edit::Portable->new;
+
+    $rw->splice(file => $file, insert => \@code, line => $start_writing);
+
+    my $sub_name;
+
+    if ($code[0] =~ /sub\s+(\w+)\s+\{/){
+        $sub_name = $1;
+    }
+    else {
+        croak "add_functionality() couldn't extract the sub name.";
+    }
+
+    my @insert = ("        $sub_name => \\&$sub_name,");
+
+    my @ret = $rw->splice(
+        file => $file, 
+        find => 'my\s+\$dt\s+=\s+\{',
+        insert => \@insert,
+    );
+
+    return 1;
 }
 sub engines {
     
@@ -418,7 +467,7 @@ sub _clean_config {
 
     my $self = shift;
     my $config_vars = shift; # href of valid params
-    my $p = shift; # href of params passed in
+    my $p = shift;           # href of params passed in
 
     for my $var (keys %$config_vars){
        
@@ -473,6 +522,7 @@ sub _config {
 
         file => 1,
         extensions => 1,
+        maxdepth => 1,
         regex => 1,
         copy => 1,
         diff => 1,
@@ -512,6 +562,9 @@ sub _config {
         file_contents => 0,
         exec => 0,
         limit => 0,
+        line_num => 0,              # inject()
+        add_functionality => 0,
+        add_functionality_prod => 0,
     );
 
     $self->{valid_params} = \%valid_params;
@@ -607,7 +660,7 @@ sub _file {
 
     if (-d $self->{params}{file}){
         $self->{params}{directory} = 1;
-        $self->{params}{extensions} = $p->{extensions} // [qw(pm pl)];
+        $self->{params}{extensions} = $p->{extensions} // [qw(*.pm *.pl)];
     }
     else {
         if (! $self->{params}{file} || ! -f $self->{params}{file}){
@@ -658,30 +711,16 @@ sub _run_directory {
     my $self = shift;
     my $p = shift;
 
-    my @files;
-
     my $dir = $self->{params}{file};
 
-    find({wanted => sub {
-            
-            trace() if $ENV{TRACE};
+    $self->{rw} = File::Edit::Portable->new;
 
-            return if ! -f;
-
-            my @extensions = @{$self->{params}{extensions}};
-            my $exts = join('|', @extensions);
-
-            if ($_ !~ /\.(?:$exts)$/i){
-                return;
-            }
-
-            my $file = $File::Find::name;
-
-            push @files, $file;
-          },
-            no_chdir => 1
-        }, $dir
-    );
+    my @files = $self->{rw}->dir(
+                            dir => $dir,
+                            maxdepth => $self->{params}{maxdepth} || 0,
+                            types => $self->{params}{extensions},
+                            list => 1,
+                        );
 
     my %struct;
 
@@ -720,12 +759,12 @@ sub _write_file {
 
     my $self = shift;
 
-    my $file = $self->{params}{file};
     my $copy = $self->{params}{copy};
+    
+    my $file = $self->{params}{file};
     my $contents = $self->{write_file_contents};
-
+   
     return if ! $file;
-
     $file = $copy if $copy;
 
     my $write_response;
@@ -1269,9 +1308,9 @@ so be careful.
 
 Only specific params are guaranteed to stay persistent throughout a run on the
 same object, and are best set in C<new()>. These parameters are C<file>,
-C<extensions>, C<cache>, C<regex>, C<copy>, C<no_indent> and C<diff>.
+C<extensions>, C<maxdepth>, C<cache>, C<regex>, C<copy>, C<no_indent> and C<diff>.
 
-Note: omit this parameter if all you'll be using is the C<module()> method.
+Note: omit the C<file> parameter if all you'll be using is the C<module()> method.
 
 
 
@@ -1429,9 +1468,14 @@ details.
 
 =head2 C<inject>
 
-Parameters: C<inject_use =E<gt> ['use Module::Name', 'use Module2::Name']> or
+Parameters (all are mutually exclusive, use only one):
+
+C<line_num =E<gt> 33> with C<code =E<gt> \@code> or,
+C<inject_use =E<gt> ['use Module::Name', 'use Module2::Name']> or,
 C<inject_after_sub_def =E<gt> ['code line 1;', 'code line 2;']>
 
+C<line_num> will inject the block of code in the array reference immediately after the line number specified.
+
 C<inject_use> will inject the statements prior to all existing C<use>
 statements that already exist in the file(s). If none are found, will inject
 the statements right after a C<Package> statement if found.
@@ -1494,8 +1538,7 @@ it easy to write new engines and for testing.
 
 =head2 C<add_functionality>
 
-WARNING!: This method is highly experimental and is used for developing
-internal processors only. Only 'engine' is functional, and only half way.
+WARNING!: This method is for development of this distribution only!
 
 While writing new processors, set the processor type to a callback within the
 local working file. When the code performs the actions you want it to, put a
@@ -1505,8 +1548,7 @@ specified processor, and configure it for use. See
 C<examples/write_new_engine.pl> for an example of creating a new 'engine'
 processor.
 
-
-
+Returns 1 on success.
 
 Parameters:
 
@@ -1578,15 +1620,20 @@ persistent until changed again.
 
 State: Persistent
 
-Default: C<[qw(pm pl)]>
+Default: C<['*.pm', '*.pl')]>
 
 By default, we load only C<*.pm> and C<*.pl> files. Use this parameter to load
 different files. Only useful when a directory is passed in as opposed to a
 file. This parameter is persistent until manually reset and should be set in
 C<new()>.
 
-Values: Array reference where each element is the name of the extension
-(less the dot). For example, C<[qw(pm pl)]> is the default.
+Values: Array reference where each element is the names of files to find. Any wildcard or regex that are valid in L<File::Find::Rule's|http://search.cpan.org/~rclamp/File-Find-Rule-0.33/lib/File/Find/Rule.pm> are valid here. For example, C<[qw(*.pm *.pl)]> is the default.
+
+
+=item C<maxdepth>
+
+When running in directory mode, how many levels deep do you want to traverse? Default is unlimited. Set to a positive integer to set.
+
 
 =item C<cache>
 
