@@ -3,7 +3,7 @@ use 5.012;
 use strict;
 use warnings;
 
-our $VERSION = '1.50';
+our $VERSION = '1.51';
 
 use Carp;
 use Data::Dumper;
@@ -35,7 +35,6 @@ sub new {
 
     return $self;
 }
-
 sub _dt {
 
     trace() if $ENV{TRACE};
@@ -52,7 +51,6 @@ sub _dt {
 
     return $dt;
 }
-
 sub exists {
 
     trace() if $ENV{TRACE};
@@ -67,7 +65,6 @@ sub exists {
         return 0;
     }
 }
-
 sub module {
 
     trace() if $ENV{TRACE};
@@ -106,7 +103,6 @@ sub module {
         return \@subs;
     };
 }
-
 sub inject {
 
     trace() if $ENV{TRACE};
@@ -119,6 +115,17 @@ sub inject {
 
         my @file_contents = @{ $p->{file_contents} };
 
+        # after line number
+
+        if ($p->{line_num}){
+            
+            splice @file_contents, $p->{line_num}, 0, @{ $p->{code} };
+
+            $p->{write_file_contents} = \@file_contents;
+        }
+
+        # inject a use statement
+
         if ($p->{inject_use}) {
 
             my $use = qr/use\s+\w+/;
@@ -143,6 +150,8 @@ sub inject {
 
         }
 
+        # inject code after sub definition
+
         if ($p->{inject_after_sub_def}) {
 
             my $code = $p->{inject_after_sub_def};
